@@ -381,8 +381,8 @@ BEGIN_METHOD(CARRAY_##_type##_push, GB_##_gtype value) \
 END_METHOD
 
 IMPLEMENT_add(integer, INTEGER)
-IMPLEMENT_add(short, INTEGER)
-IMPLEMENT_add(byte, INTEGER)
+/*IMPLEMENT_add(short, INTEGER)
+IMPLEMENT_add(byte, INTEGER)*/
 IMPLEMENT_add(float, FLOAT)
 IMPLEMENT_add(date, DATE)
 IMPLEMENT_add(string, STRING)
@@ -399,9 +399,18 @@ BEGIN_METHOD(CARRAY_##_type##_put, GB_##_gtype value; GB_INTEGER index) \
   \
 END_METHOD
 
+#define IMPLEMENT_put2(_type, _gtype, _gstore) \
+BEGIN_METHOD(CARRAY_##_type##_put, GB_##_gtype value; GB_INTEGER index) \
+\
+  void *data = get_data(THIS, VARG(index)); \
+  if (!data) return; \
+  GB_Store(GB_T_##_gstore, (GB_VALUE *)ARG(value), data); \
+  \
+END_METHOD
+
 IMPLEMENT_put(integer, INTEGER)
-IMPLEMENT_put(short, INTEGER)
-IMPLEMENT_put(byte, INTEGER)
+IMPLEMENT_put2(short, INTEGER, SHORT)
+IMPLEMENT_put2(byte, INTEGER, BYTE)
 IMPLEMENT_put(float, FLOAT)
 IMPLEMENT_put(date, DATE)
 IMPLEMENT_put(string, STRING)
@@ -710,7 +719,7 @@ PUBLIC GB_DESC NATIVE_ShortArray[] =
   
   GB_METHOD("Add", NULL, CARRAY_integer_add, "(Value)i[(Index)i]"),
   GB_METHOD("Push", NULL, CARRAY_integer_push, "(Value)i"),
-  GB_METHOD("_put", NULL, CARRAY_integer_put, "(Value)i(Index)i"),
+  GB_METHOD("_put", NULL, CARRAY_short_put, "(Value)i(Index)i"),
   GB_METHOD("Find", "i", CARRAY_integer_find, "(Value)i[(Mode)i]"),
 
   GB_METHOD("Pop", "i", CARRAY_pop, NULL),
@@ -735,7 +744,7 @@ PUBLIC GB_DESC NATIVE_ByteArray[] =
   
   GB_METHOD("Add", NULL, CARRAY_integer_add, "(Value)i[(Index)i]"),
   GB_METHOD("Push", NULL, CARRAY_integer_push, "(Value)i"),
-  GB_METHOD("_put", NULL, CARRAY_integer_put, "(Value)i(Index)i"),
+  GB_METHOD("_put", NULL, CARRAY_byte_put, "(Value)i(Index)i"),
   GB_METHOD("Find", "i", CARRAY_integer_find, "(Value)i[(Mode)i]"),
 
   GB_METHOD("Pop", "i", CARRAY_pop, NULL),
